﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Xml.Serialization;

namespace CheckHobbyProcess
{
	public class AppConfig : INotifyPropertyChanged, IDataErrorInfo
	{
		public const string FILE_EXTENSION = ".conf";
		public const string FILE_FOLDER_SWITCH = "useAppData";
		public const string STOP_BUTTON_TEXT = "アプリを止める";
		public const string STOP_BUTTON_TOOLTIP = "このボタンを押すと対象のアプリケーションを終了します。\n強制終了ではないので場合によっては終了できません。\nその場合は右の猶予ボタンを押してから正常に終わらせてください。";
		public const string EXTEND_BUTTON_TEXT = "猶予";
		public const string EXTEND_BUTTON_TOOLTIP = "次の表示まで少しだけ時間を置きます";

		public static AppConfig Current { get; set; }
		public static string FilePath { get; set; }

		static AppConfig()
		{
			Current = new AppConfig();

			Current.TargetProcessNames = new List<string>()
			{
				"MineSweeper",
				"Mahjong",
				"SpiderSolitaire"
			};
			Current.TargetPathNames = new List<string>()
			{
				@"C:\Program Files\Microsoft Games\"
			};

			Current.ExtensionOfTime = 10;
			Current.Interval = 15;

			Current.CheckPeriodOfTime = new bool[][]
			{
				new bool[]{true, true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, true, true, true},
				new bool[]{true, true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, true},
				new bool[]{true, true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, true},
				new bool[]{true, true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, true},
				new bool[]{true, true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, true},
				new bool[]{true, true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false},
				new bool[]{true, true, true, true, true, true, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false, false}
			};

			Current.UntouchableProcessNames = new string[]
			{
				"taskmgr",
				"explorer",
				"winlogon",
				"services",
				"svchost",
				"System",
				"wininit",
				"system32",
				"lsass",
				"smss",
				"csrss",
				"wmpnetwk",
				"wisptis",
				"spoolsv",
				"lsm",
				"SearchFilterHost",
				"SearchIndexer",
				"SearchProtocolHost",
				"Idle"
			};

			Current.StopButtonText = STOP_BUTTON_TEXT;
			Current.StopButtonTooltip = STOP_BUTTON_TOOLTIP;
			Current.ExtendButtonText = EXTEND_BUTTON_TEXT;
			Current.ExtendButtonTooltip = EXTEND_BUTTON_TOOLTIP;
		}

		private List<string> _targetProcessNames;
		public List<string> TargetProcessNames
		{
			get
			{
				return _targetProcessNames;
			}
			set
			{
				_targetProcessNames = (value == null ? new List<string>() : value);
			}
		}

		private List<string> _targetPathNames;
		public List<string> TargetPathNames
		{
			get
			{
				return _targetPathNames;
			}
			set
			{
				_targetPathNames = (value == null ? new List<string>() : value);
			}
		}

		// 分単位
		private int _extensionOfTime;
		public int ExtensionOfTime
		{
			get
			{
				return _extensionOfTime;
			}
			set
			{
				_extensionOfTime = value;
				NotifyPropertyChanged("ExtensionOfTime");
				if (value < 1 || 120 < value)
				{
					Errors["ExtensionOfTime"] = "チェック間隔は1～120分の間に設定してください。";
				}
				else
				{
					Errors.Remove("ExtensionOfTime");
				}
			}
		}

		// 秒単位
		private int _interval;
		public int Interval
		{
			get
			{
				return _interval;
			}
			set
			{
				_interval = value;
				NotifyPropertyChanged("Interval");
				if (value < 1 || 300 < value)
				{
					Errors["Interval"] = "チェック間隔は1～300秒の間に設定してください。";
				}
				else
				{
					Errors.Remove("Interval");
				}
			}
		}

		// テキストボックスで編集できるようにTargetProcessNamesと
		// TargetPathNamesのバインドをこのプロパティで受け付ける。
		// 単純に文字列を比較するだけなので詳細なチェックはしない。
		[XmlIgnore]
		public string TargetListString
		{
			get
			{
				string res = "";

				foreach (string processStr in TargetProcessNames)
					res += processStr + '\n';
				foreach (string pathStr in TargetPathNames)
					res += pathStr + '\n';

				return res;
			}
			set
			{
				TargetProcessNames.Clear();
				TargetPathNames.Clear();
				if (value == null)
					return;

				string[] lines = value.Split('\n');
				foreach (string lineTmp in lines)
				{
					string line = lineTmp.Trim();
					if (0 < line.Length)
					{
						if (0 <= line.IndexOf('\\')) // \が含まれていたらディレクトリ
						{
							TargetPathNames.Add(line);
						}
						else
						{
							TargetProcessNames.Add(line);
						}
					}
				}
			}
		}

		// システムプロセスなどの終了してはいけない、チェック不要のプロセス名リスト。
		private string[] _untouchableProcessNames;
		public string[] UntouchableProcessNames
		{
			get
			{
				return _untouchableProcessNames;
			}
			set
			{
				_untouchableProcessNames = (value == null ? new string[0] : value);
			}
		}

		// [曜日,時間]、サイズは[7,24]
		// 曜日はSystem.DayOfWeekをintにキャストしたものをindexとする。
		// (日曜が0)
		// trueで検出開始
		private bool[][] _checkPeriodOfTime;
		public bool[][] CheckPeriodOfTime
		{
			get
			{
				return _checkPeriodOfTime;
			}
			set
			{
				if (value == null || value.Length != 7)
					throw new ArgumentException("CheckPeriodOfTimeの行数違反");
				foreach (bool[] row in value)
				{
					if (row == null || row.Length != 24)
					{
						throw new ArgumentException("CheckPeriodOfTimeの列数違反");
					}
				}
				_checkPeriodOfTime = value;
			}
		}

		private string _stopButtonText;
		public string StopButtonText
		{
			get
			{
				return _stopButtonText;
			}
			set
			{
				if (value == null || value.Trim().Length == 0)
					_stopButtonText = STOP_BUTTON_TEXT;
				else
					_stopButtonText = value.Trim();
			}
		}
		private string _stopButtonTooltip;
		public string StopButtonTooltip
		{
			get
			{
				return _stopButtonTooltip;
			}
			set
			{
				if (value == null || value.Trim().Length == 0)
					_stopButtonTooltip = STOP_BUTTON_TOOLTIP;
				else
					_stopButtonTooltip = value.Trim();
			}
		}

		private string _extendButtonText;
		public string ExtendButtonText
		{
			get
			{
				return _extendButtonText;
			}
			set
			{
				if (value == null || value.Trim().Length == 0)
					_extendButtonText = EXTEND_BUTTON_TEXT;
				else
					_extendButtonText = value.Trim();
			}
		}

		private string _extendButtonTooltip;
		public string ExtendButtonTooltip
		{
			get
			{
				return _extendButtonTooltip;
			}
			set
			{
				if (value == null || value.Trim().Length == 0)
					_extendButtonTooltip = EXTEND_BUTTON_TOOLTIP;
				else
					_extendButtonTooltip = value.Trim();
			}
		}

		public AppConfig()
		{
			TargetPathNames = new List<string>();
			TargetProcessNames = new List<string>();
			UntouchableProcessNames = new string[0];

			Errors = new Dictionary<string, string>();
		}

		public AppConfig Copy()
		{
			Debug.Assert(
				this._checkPeriodOfTime != null &&
				this._targetPathNames != null &&
				this._targetProcessNames != null &&
				this._untouchableProcessNames != null
			);

			AppConfig res = (AppConfig)this.MemberwiseClone();

			res._targetProcessNames = new List<string>(this._targetProcessNames);
			res._targetPathNames = new List<string>(this._targetPathNames);

			res._untouchableProcessNames = new string[this._untouchableProcessNames.Length];
			this._untouchableProcessNames.CopyTo(res._untouchableProcessNames, 0);

			int rowNum = this._checkPeriodOfTime.Length;
			int colNum = this._checkPeriodOfTime[0].Length;
			res._checkPeriodOfTime = new bool[rowNum][];
			for(int i = 0; i < rowNum; i++)
			{
				res._checkPeriodOfTime[i] = new bool[colNum];
				this._checkPeriodOfTime[i].CopyTo(res._checkPeriodOfTime[i], 0);
			}

			return res;
		}

		#region プロパティ変更通知

		public event PropertyChangedEventHandler PropertyChanged;

		protected virtual void NotifyPropertyChanged(string property)
		{
			if (this.PropertyChanged != null)
			{
				this.PropertyChanged(this, new PropertyChangedEventArgs(property));
			}
		}
		#endregion // プロパティ変更通知

		#region 入力値のエラーチェック関係

		[System.Xml.Serialization.XmlIgnoreAttribute]
		public virtual Dictionary<string, string> Errors { get; private set; }

		public string Error
		{
			get
			{
				int c = Errors.Count;
				if (c == 0)
					return null;
				else
					return c + "個のエラーがあります。";
			}
		}

		public string this[string columnName]
		{
			get
			{
				if (Errors.ContainsKey(columnName))
					return Errors[columnName];

				return null;
			}
		}

		public bool HasError()
		{
			return Errors.Count != 0;
		}

		#endregion // 入力値のエラーチェック関係

		#region ファイル操作

		// 例外処理は呼び出し元で
		public static void Save()
		{
			if (FilePath != null)
			{
				// ロードしたパス or 最後に書き込んだパスを使用。
				SaveActually(FilePath);
			}
			else
			{
				string pathExe, pathAppData;
				GetFilePath(out pathExe, out pathAppData);

				// ロードしたパス or 最後に書き込んだパスがない場合
				// path1 → path2の順に書き込んでみて
				// 実際に書き込めたパスをFilePathに残す。
				try
				{
					SaveActually(pathExe);
					FilePath = pathExe;
				}
				catch
				{
					SaveActually(pathAppData);
					FilePath = pathAppData;
				}
			}
		}

		private static void SaveActually(string path)
		{
			string dir = new FileInfo(path).Directory.FullName;
			if (!Directory.Exists(dir))
			{
				Directory.CreateDirectory(dir);
			}

			XmlSerializer serializer = new XmlSerializer(typeof(AppConfig));
			using (StreamWriter sout = new StreamWriter(path))
			{
				serializer.Serialize(sout, Current);
			}
		}

		// 例外処理は呼び出し元で
		public static void Load()
		{
			string pathExe, pathAppData;
			GetFilePath(out pathExe, out pathAppData);

			// ファイルがあったら、例え読み込めなかったとしてもそれをFilePathに残す。
			if (File.Exists(pathExe))
			{
				FilePath = pathExe;
				LoadActually(pathExe);
			}
			else if (File.Exists(pathAppData))
			{
				FilePath = pathAppData;
				LoadActually(pathAppData);
			}
			else
			{
				throw new Exception("AppConfig : Load失敗");
			}
		}

		private static void LoadActually(string path)
		{
			XmlSerializer serializer = new XmlSerializer(typeof(AppConfig));
			using (StreamReader sin = new StreamReader(path))
			{
				AppConfig.Current = serializer.Deserialize(sin) as AppConfig;
			}
		}

		/// <summary>
		/// 設定ファイルのパス候補を得る。
		/// </summary>
		/// <param name="pathExe">
		/// パス候補1。
		/// 実行ファイルと同じフォルダ。
		/// 次の場合、nullを返す。
		/// 1.そのディレクトリにFILE_DIR_SWITCHがある場合。
		/// 2.ProgramFilesの子フォルダの場合。
		/// </param>
		/// <param name="pathExe">パス候補2。ローミングユーザーのアプリケーションデータが保存されているディレクトリ。 </param>
		private static void GetFilePath(out string pathExe, out string pathAppData)
		{
			FileInfo fileInfo = new FileInfo(Environment.GetCommandLineArgs()[0]);
			string fileExt = fileInfo.Extension;
			string fileName = fileInfo.Name;
			string exeName = fileName.Substring(0, fileName.Length - fileExt.Length);

			pathExe = fileInfo.Directory.FullName;
			if (File.Exists(pathExe + "\\" + FILE_FOLDER_SWITCH)
				|| pathExe.StartsWith(Environment.GetFolderPath(Environment.SpecialFolder.ProgramFilesX86))
				|| pathExe.StartsWith(Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles)))
			{
				pathExe = null;
			}
			else
			{
				pathExe += "\\" + exeName + FILE_EXTENSION;
			}

			pathAppData = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData)
						+ "\\" + exeName + "\\" + exeName + FILE_EXTENSION;
		}

		#endregion // ファイル操作
	}
}
