﻿using System.Diagnostics;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Data;
using System.Windows.Input;
using System.Windows.Media;

namespace CheckHobbyProcess
{
	/// <summary>
	/// ConfigWindow.xaml の相互作用ロジック
	/// </summary>
	public partial class ConfigWindow : Window
	{
		private const int CHECK_TOGGLE_BUTTONS_AT_ONCE = 6;

		private ToggleButton[][] _btnTable;

		public ConfigWindow()
		{
			InitializeComponent();
		}

		protected virtual void OnLoaded(object sender, RoutedEventArgs e)
		{
			AppConfig conf = AppConfig.Current.Copy();
			Debug.Assert(conf != null && !conf.HasError() );
			this.DataContext = conf;
			InitializeScheduleGrid(conf);
		}

		protected virtual void OnOkButtonClick(object sender, RoutedEventArgs e)
		{
			if (!IsValid(this))
			{
				MessageBox.Show(this, "不正な設定項目があります。", "確認", MessageBoxButton.OK, MessageBoxImage.Information);
				return;
			}

			AppConfig.Current = (AppConfig)this.DataContext;
			DialogResult = true;
		}

		protected virtual void OnCancelButtonClick(object sender, RoutedEventArgs e)
		{
			DialogResult = false;
		}

		protected virtual void OnScheduleButtonClick(object sender, RoutedEventArgs e)
		{
			ModifierKeys mk = Keyboard.Modifiers;
			if (mk == ModifierKeys.None)
				return;

			AppConfig conf = (AppConfig)this.DataContext;
			bool[][] checkPeriodOfTime = conf.CheckPeriodOfTime;
			int rowNum = checkPeriodOfTime.Length;
			int colNum = checkPeriodOfTime[0].Length;

			ToggleButton btnSrc = (ToggleButton)e.Source;
			ButtonPosition p = (ButtonPosition)btnSrc.Tag;

			bool chked = btnSrc.IsChecked ?? false;
			if ((mk & ModifierKeys.Control) != 0 && (mk & ModifierKeys.Shift) != 0 && p.Col < 24)
			{
				// 横1列
				ToggleButton[] rowBtns = _btnTable[p.Row];
				for (int col = 0; col < colNum; col++)
				{
					rowBtns[col].IsChecked = chked;
				}
			}
			else if (mk == ModifierKeys.Shift)
			{
				// 縦1列
				for (int row = 0; row < rowNum; row++)
				{
					_btnTable[row][p.Col].IsChecked = chked;
				}
			}
			else if (mk == ModifierKeys.Control)
			{
				// チェックされたものも含めて右?つを変更
				ToggleButton[] rowBtns = _btnTable[p.Row];
				for (int i = 1; i < CHECK_TOGGLE_BUTTONS_AT_ONCE; i++)
				{
					int col = p.Col + i;
					if (_btnTable[0].Length <= col)
					{
						break;
					}
					rowBtns[col].IsChecked = chked;
				}
			}
		}

		private void OnArrangeAllChecksButtunClick(object sender, RoutedEventArgs e)
		{
			ArrangeChecks(0, 1, 6);
		}

		private void OnArrangeChecksOfWeekdayButtunClick(object sender, RoutedEventArgs e)
		{
			ArrangeChecks(1, 2, 5);
		}

		// 即値でレイアウト
		private void InitializeScheduleGrid(AppConfig conf)
		{
			RowDefinitionCollection rowDefs = scheduleGrid.RowDefinitions;
			ColumnDefinitionCollection colDefs = scheduleGrid.ColumnDefinitions;

			Brush brushRow = new SolidColorBrush(Color.FromRgb(255, 255, 204));
			Brush brushCol = new SolidColorBrush(Color.FromRgb(244, 246, 250));
			Brush brushRC = new SolidColorBrush(Color.FromRgb(244, 247, 203));
			Brush brushSun = new SolidColorBrush(Color.FromRgb(248, 235, 232));
			Brush brushSunC = new SolidColorBrush(Color.FromRgb(239, 228, 228));

			bool[][] checkPeriodOfTime = conf.CheckPeriodOfTime;
			int rowNum = checkPeriodOfTime.Length;
			int colNum = checkPeriodOfTime[0].Length;

			// colNumは全体で24時間を表す。その1/4は6時間を表す。6時間だけダブって表示させる。
			int colPlusNum = colNum / 4;
			int allColNum = colNum + colPlusNum;

			for (int i = 0; i < rowNum + 1; i++)
			{
				RowDefinition rowDef = new RowDefinition();
				rowDef.Height = new GridLength(24);
				rowDefs.Add(rowDef);
			}
			for (int i = 0; i < allColNum + 3; i++)
				colDefs.Add(new ColumnDefinition());
			colDefs[0].Width = new GridLength(48);
			for (int i = 0; i < allColNum + 1; i++)
				colDefs[i + 1].Width = new GridLength(16);

			string toolTip =
				"Ctrlを押しながらクリックで、クリックしたところから" + CHECK_TOGGLE_BUTTONS_AT_ONCE + "マス変更。\n" +
				"Shiftを押しながらクリックで縦1列を変更。\n" +
				"Ctrl+Shiftを押しながらクリックで横1行を変更(0～24時の範囲)。";

			// 横軸「時刻」ラベルの作成
			string[] timeStr = new string[] { "0時", "6時", "12時", "18時", "24時", "30時" };
			for (int i = 0; i < timeStr.Length; i++)
			{
				Label lblTime = new Label();
				lblTime.Content = timeStr[i];
				lblTime.ToolTip = toolTip;
				Grid.SetRow(lblTime, 0);
				Grid.SetColumn(lblTime, i * 6 + 1);
				Grid.SetColumnSpan(lblTime, 6);
				scheduleGrid.Children.Add(lblTime);
			}

			// 縦軸「曜日」ラベルの作成
			string[] weekStr = new string[]{"日","月","火","水","木","金","土"};
			for(int i = 0; i < rowNum; i++)
			{
				Label lblWeek = new Label();
				lblWeek.Content = weekStr[i] + "曜日";
				lblWeek.ToolTip = toolTip;
				Grid.SetRow(lblWeek, i + 1);
				Grid.SetColumn(lblWeek, 0);
				Grid.SetColumnSpan(lblWeek, 2);
				scheduleGrid.Children.Add(lblWeek);
			}

			// 時間帯設定ボタンの作成
			ToggleButton[][] btnTable = new ToggleButton[rowNum][]; // バインド作成用
			_btnTable = btnTable;
			for (int row = 0; row < rowNum; row++)
			{
				btnTable[row] = new ToggleButton[allColNum];
				for (int col = 0; col < allColNum; col++)
				{
					ToggleButton btn = new ToggleButton();
					Grid.SetRow(btn, row + 1);
					Grid.SetColumn(btn, col + 2);

					if((row == 0 && col < 24) || (row == 6 && 24 <= col))
					{
						btn.Background = (col / 6 % 2 == 0 ? brushSun : brushSunC);
					}
					else if ((row % 2 == 1) ^ (24 <= col))
					{
						btn.Background = (col / 6 % 2 == 0 ? Brushes.White : brushCol);
					}
					else
					{
						btn.Background = (col / 6 % 2 == 0 ? brushRow : brushRC);
					}
					btn.ToolTip = "" + col + "時";
					btn.Tag = new ButtonPosition() { Row = row, Col = col };
					btn.Click += OnScheduleButtonClick;
					scheduleGrid.Children.Add(btn);
					btnTable[row][col] = btn;
				}
			}
			
			// 24時間分のバインドを設定
			for (int row = 0; row < rowNum; row++)
			{
				for (int col = 0; col < colNum; col++)
				{
					ToggleButton btn = btnTable[row][col];

					Binding bind = new Binding();
					bind.Mode = BindingMode.TwoWay;
					bind.Path = new PropertyPath(string.Format("CheckPeriodOfTime[{0}][{1}]", row, col));
					btn.SetBinding(ToggleButton.IsCheckedProperty, bind);
				}
			}

			// 追加6時間分のバインドを設定。24～30時を翌日の0～6時にバインド。
			for (int row = 0; row < rowNum; row++)
			{
				for (int col = 0; col < colPlusNum; col++)
				{
					int nextRow = row + 1;
					if (rowNum <= nextRow)
					{
						nextRow = 0;
					}

					ToggleButton btn = btnTable[row][colNum + col];
					ToggleButton btnSrc = btnTable[nextRow][col];

					Binding bind = new Binding();
					bind.Mode = BindingMode.TwoWay;
					bind.Source = btnSrc;
					bind.Path = new PropertyPath("IsChecked");
					btn.SetBinding(ToggleButton.IsCheckedProperty, bind);
				}
			}
		}

		private bool IsValid(DependencyObject node)
		{
			if (node != null)
			{
				if (Validation.GetHasError(node))
				{
					return false;
				}
			}

			foreach (object subnode in LogicalTreeHelper.GetChildren(node))
			{
				if (subnode is DependencyObject)
				{
					if (!IsValid((DependencyObject)subnode))
					{
						return false;
					}
				}
			}

			return true;
		}

		// スケジュール用ToggleButton配列のコピー
		// rowSrc曜日のスケジュールをrowFrom～rowTo曜日にコピーする
		private void ArrangeChecks(int rowSrc, int rowFrom, int rowTo)
		{
			AppConfig conf = (AppConfig)this.DataContext;
			int colNum = conf.CheckPeriodOfTime[0].Length;

			ToggleButton[] src = _btnTable[rowSrc];
			for (int row = rowFrom; row <= rowTo; row++)
			{
				for (int col = 0; col < colNum; col++)
				{
					_btnTable[row][col].IsChecked = src[col].IsChecked;
				}
			}
		}

		// スケジュール用ToggleButtonの位置(Row=曜日、Col=時間)
		private class ButtonPosition
		{
			public int Row;
			public int Col;
		}
	}
}
